#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import sys, os
import subprocess

# When no delay is specified in a gif file, 100ms is default
DEFAULT_DELAY_MS = 100

input_file = None
output_file = None

try:
    input_file = sys.argv[1]
    output_file = "%s.xml" % (input_file.split(".")[0],)
except:
    print "Usage: %s <input.gif>" % (sys.argv[0],)
    sys.exit(1)

    
class Anim(object):
    def __init__(self):
        self.content = []
        
    def add_header(self):
        self.content.append('<?xml version="1.0" encoding="utf-8"?>\n')
        self.content.append('<animation-list\n')
        self.content.append('    xmlns:android="http://schemas.android.com/apk/res/android"\n')
        self.content.append('    android:oneshot="false">\n')
        
    def add_item(self, resource, duration):
        self.content.append(
            '    <item android:drawable="@drawable/%s" android:duration="%d" />\n' %
            (resource, duration))
        
    def add_footer(self):
        self.content.append('</animation-list>\n')
        
    def write_file(self):
        file_descriptor = open(output_file, "w")
        file_descriptor.writelines(self.content)
        file_descriptor.close()

    
class Gif(object):
    def __init__(self, gif):
        self.input_file = gif
        gif_info = subprocess.Popen(
            ["gifsicle", "--info", input_file],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

        (gif_info_out, gif_info_error) = gif_info.communicate()
        
        self.gif_info_lines = gif_info_out.splitlines()
        
    def get_info(self):
        return self.gif_info_lines
                
    def explode(self):    
        return_code = subprocess.call(
            ["gifsicle", "--explode", "--unoptimize", self.input_file])
        
        return not return_code
        
    def to_png(self, gif_frame):
        return_code = subprocess.call(["gif2png", "-d", "-s", "-O", gif_frame])
        
        return return_code


def main():
    print "Processing %s..." % (input_file,)
    
    anim = Anim()
    anim.add_header() 
        
    gif = Gif(input_file)

    if not gif.explode():
        print "Error exploding gif input file %s" % (input_file,)
        #sys.exit(1)

    info_lines = gif.get_info()

    counter = 0
    for line in info_lines:
        line = line.strip()

        if line.startswith("disposal"):
            line_items = line.split()
            
            if len(line_items) == 4:
                milliseconds = int(round(float(line_items[3][:-1]) * 1000))
            else:
                milliseconds = DEFAULT_DELAY_MS
        
            # Rename and convert gif to optimized png
            renamed_gif = "%s_%03d.gif" % (input_file.split(".")[0], counter)
            os.rename("%s.%03d" % (input_file, counter), renamed_gif)
            result = gif.to_png(renamed_gif)
            
            anim.add_item(renamed_gif.split(".")[0], milliseconds)
            counter = counter + 1
    
    anim.add_footer()
    anim.write_file()
    
    os.unlink(input_file)
    sys.exit(0)

if __name__ == "__main__":
    main()
    
